#include "delay.h"

void delay_ms(uint16_t delay_ms)
{
	while (delay_ms > 125)
	{
		RTCEN = 1; 	/* Enable Input Clock Supply */

		ITIF = 0;			/* Clear 12-bit Interval Timer Interrupt Flag */
		ITMK = 1;			/* Mask Interval Timer Interrupt */
		ITMC &= 0x0FFF;		/* Stop Count Operation by clearing RINTE bit of ITMC Register */

		ITMC = 4095;	/* Loading 125ms value to the ITCMP11 to ITCMP0 bits of the ITMC Register */

		ITMC |= 0x8000;		/* Start Count Operation by setting RINTE bit of ITMC Register */

		while (ITIF == 0)	/* Polling the ITIF flag */
		{
			NOP();
		}

		ITIF = 0;			/* Clear 12-bit Interval Timer Interrupt Flag */

		RTCEN = 0;			/* Disable Input Clock Supply */

		delay_ms = delay_ms - 125;
	}

	delay_ms = ((delay_ms * 32768)/1000) - 1;

	RTCEN = 1; 	/* Enable Input Clock Supply */

	ITIF = 0;			/* Clear 12-bit Interval Timer Interrupt Flag */
	ITMK = 1;			/* Mask Interval Timer Interrupt */
	ITMC &= 0x0FFF;		/* Stop Count Operation by clearing RINTE bit of ITMC Register */

	ITMC = delay_ms;	/* Loading delay_ms remaining value to the ITCMP11 to ITCMP0 bits of the ITMC Register */

	ITMC |= 0x8000;		/* Start Count Operation by setting RINTE bit of ITMC Register */

	while (ITIF == 0)	/* Polling the ITIF flag */
	{
		NOP();
	}

	ITIF = 0;			/* Clear 12-bit Interval Timer Interrupt Flag */

	RTCEN = 0;			/* Disable Input Clock Supply */
}

void delay_us_TAU_Init(void)
{
    TAU0EN = 1U;    	/* Supplies Input Clock */

    TPS0 = 0x0000U |	/* CKm0 = fCLK */
    	   0x0000U |	/* CKm1 = fCLK */
		   0x0000U |	/* CKm2 = fCLK/2^1 */
		   0x0000U;		/* CKm3 = fCLK/2^8 */

    /* Stop all channels */
    TT0 = 0x0001U |		/* Channel 0 (TTm0) operation is stopped (stop trigger is generated) */
    	  0x0002U |		/* Channel 1 (TTm1) operation is stopped (stop trigger is generated) */
		  0x0004U |		/* Channel 2 (TTm2) operation is stopped (stop trigger is generated) */
		  0x0008U |		/* Channel 3 (TTm3) operation is stopped (stop trigger is generated) */
		  0x0200U |		/* Channel 1 higher 8 bits (TTHm1) operation is stopped (stop trigger is generated) */
		  0x0800U;		/* Channel 3 higher 8 bits (TTHm3) operation is stopped (stop trigger is generated) */

    /* Mask Channel 0 Interrupt */
    TMMK00 = 1U;    /* Disable INTTM00 Interrupt */
    TMIF00 = 0U;    /* Clear INTTM00 Interrupt Flag */

    /* Channel 0 used as Interval Timer */
    TMR00 = 0x0000U |	/* Selection of macro clock (fMCK) of Channel 0 (CKSmn1 and CKSmn0 bits). Selected CKm0. */
    		0x0000U	|	/* Selection of count clock (CCK) of channel n (CCSmn bit). Selected fMCK specified by CKSmn1 and CKSmn0 bits. */
			0x0000U |	/* Bit 11 of Channel 0 TMR00 Register. Fixed to 0. */
			0x0000U	|	/* Setting of start trigger or capture trigger of Channel 0 (STSmn2, STSmn1, and STSmn0 bits). Selected "Only software trigger start is valid (other trigger sources are unselected)". */
			0x0000U |	/* Operation mode of Channel 0 (MDmn3, MDmn2, and MDmn1 bits). Selected Interval Timer Mode. */
			0x0000U;	/* Setting of starting counting and interrupt (MDmn0 bit). Selected "interrupt is not generated when counting is started". */

    TO0 &= ~0x0001U;	/* Sets Channel 0 timer output value (TOm0) to 0 or LOW. */
    TOE0 &= ~0x0001U;	/* Disables Channel 0 timer output (TOEm0). */
}

void delay_us(uint16_t delay_us)
{
	if (delay_us < 5)
	{
		NOP();
		return;
	}
	else
	{
		while (delay_us > 2048)
		{
			TDR00 = 65535;		/* 65535 is equal to "(2048 * 32) - 1". */
			TS0 |= 0x0001U;		/* Operation enable (start) trigger of Channel 0 (TSm0). Operation is enabled (start trigger is generated). */

			while (TMIF00 != 1)	{	}	/* Wait until the INTTM00 Interrupt Flag is set. */

			TT0 |= 0x0001U;		/* Operation stop trigger of Channel 0 (TTm0). Operation is stopped (stop trigger is generated). */
			TMIF00 = 0;			/* Clear the INTTM00 Interrupt Flag. */

			delay_us = delay_us - 2048;		/* Subtract 2048 to the current delay_us value. */
		}

		if (delay_us < 5)
		{
			NOP();
			return;
		}

		TDR00 = ((delay_us - 4) * 32) - 1;
		TS0 |= 0x0001U; 	/* Operation enable (start) trigger of Channel 0 (TSm0). Operation is enabled (start trigger is generated). */

		while (TMIF00 != 1)	{	}	/* Wait until the INTTM00 Interrupt Flag is set. */

		TT0 |= 0x0001U;		/* Operation stop trigger of Channel 0 (TTm0). Operation is stopped (stop trigger is generated). */
		TMIF00 = 0;			/* Clear the INTTM00 Interrupt Flag. */

		/* Added 12 NOP() to round off the delay. */
		NOP();NOP();NOP();NOP();NOP();NOP();
		NOP();NOP();NOP();NOP();NOP();NOP();
	}
}

void delay_1us(void)
{
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
	NOP();
}

void delay_2us(void)
{
	uint8_t i;
	for(i=0;i<2;i++)
	{
		NOP();
		NOP();
		NOP();
		NOP();
		NOP();
	}
}

void delay_3us(void)
{
	uint8_t i;
	for(i=0;i<5;i++)
	{
		NOP();
		NOP();
	}
	NOP();
	NOP();
}

void delay_4us(void)
{
	uint8_t i;
	for(i=0;i<7;i++)
	{
		NOP();
		NOP();
		NOP();
	}
	NOP();
	NOP();
	NOP();
}